var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _BaseAction_id, _BaseAction_type, _BaseAction_parameters, _BaseAction_instance;
import { ELEMENT_KEY } from '../../constants.js';
let actionIds = 0;
export default class BaseAction {
    constructor(instance, type, params) {
        this.instance = instance;
        _BaseAction_id.set(this, void 0);
        _BaseAction_type.set(this, void 0);
        _BaseAction_parameters.set(this, void 0);
        _BaseAction_instance.set(this, void 0);
        this.sequence = [];
        __classPrivateFieldSet(this, _BaseAction_instance, instance, "f");
        __classPrivateFieldSet(this, _BaseAction_id, params?.id || `action${++actionIds}`, "f");
        __classPrivateFieldSet(this, _BaseAction_type, type, "f");
        __classPrivateFieldSet(this, _BaseAction_parameters, params?.parameters || {}, "f");
    }
    toJSON() {
        return {
            id: __classPrivateFieldGet(this, _BaseAction_id, "f"),
            type: __classPrivateFieldGet(this, _BaseAction_type, "f"),
            parameters: __classPrivateFieldGet(this, _BaseAction_parameters, "f"),
            actions: this.sequence
        };
    }
    /**
     * Inserts a pause action for the specified device, ensuring it idles for a tick.
     * @param duration idle time of tick
     */
    pause(duration) {
        this.sequence.push({ type: 'pause', duration });
        return this;
    }
    /**
     * Perform action sequence
     * @param skipRelease set to true if `releaseActions` command should not be invoked
     */
    async perform(skipRelease = false) {
        /**
         * transform chainable / not resolved elements into WDIO elements
         */
        for (const seq of this.sequence) {
            /**
             * continue if we don't deal with origin or elements within
             * origin at all
             */
            if (!seq.origin || typeof seq.origin === 'string') {
                continue;
            }
            /**
             * resolve promise element
             */
            if (typeof seq.origin.then === 'function') {
                await seq.origin.waitForExist();
                seq.origin = await seq.origin;
            }
            if (!seq.origin[ELEMENT_KEY]) {
                throw new Error(`Couldn't find element for "${seq.type}" action sequence`);
            }
            seq.origin = { [ELEMENT_KEY]: seq.origin[ELEMENT_KEY] };
        }
        await __classPrivateFieldGet(this, _BaseAction_instance, "f").performActions([this.toJSON()]);
        if (!skipRelease) {
            await __classPrivateFieldGet(this, _BaseAction_instance, "f").releaseActions();
        }
    }
}
_BaseAction_id = new WeakMap(), _BaseAction_type = new WeakMap(), _BaseAction_parameters = new WeakMap(), _BaseAction_instance = new WeakMap();
